package com.newxtc;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.Properties;

import javax.servlet.ServletContextEvent;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 短信参数类
 */
public class NxtInit {
	private final static Logger logger = LoggerFactory.getLogger(NxtInit.class);

	private Properties prop = null;
	private StringBuffer desJs = null;
	private StringBuffer fpJs = null;
	private static NxtInit nxtInit = null;

	public synchronized static NxtInit getInstance() {
		if (nxtInit == null) {
			nxtInit = new NxtInit();
		}
		return nxtInit;
	}

	/**
	 * 标准初始化方式 设备指纹 + 防火墙接口
	 * 
	 * @param configPath
	 */
	public void init(String configPath) {
		Object obj = load(configPath, "newxtc.ini");
		prop = (obj != null) ? (Properties) obj : null;
		obj = load(configPath, "tripledes.js");
		desJs = (obj != null) ? (StringBuffer) obj : null;
		obj = load(configPath, "fp.js");
		fpJs = (obj != null) ? (StringBuffer) obj : null;
	}

	public String getIni(String key) {
		String value = prop.getProperty(key);
		return value;
	}

	public String getDesJs() {
		return desJs.toString();
	}

	public String getfPJs() {
		return fpJs.toString();
	}

	public String getUrl(String key) {
		String url = prop.getProperty(key);
		if (url != null && url.indexOf("http") >= 0) {
			if (!url.endsWith("/")) {
				url = url + "/";
			}
			return url;
		} else {
			return null;
		}
	}

	public void contextDestroyed(ServletContextEvent event) {
	}

	private Object load(String configPath, String configName) {
		InputStream inStream = null;
		try {
			if (configPath == null) {
				inStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(configName);
			} else {
				String dataConfigPath = (!configPath.matches(".*[\\/]")) ? configPath + File.separator + configName : configPath + configName;
				inStream = new FileInputStream(dataConfigPath);
			}
			if ("newxtc.ini".equals(configName)) {
				Properties prop = new Properties();
				prop.load(inStream);
				System.out.println(" Config Ini(" + (configPath != null ? configPath : configName) + ") init finish!");
				Iterator<Object> it = prop.keySet().iterator();
				while (it.hasNext()) {
					String key = (String) it.next();
					String value = prop.getProperty(key);
					System.out.println("  |--" + key + "=" + value);
				}
				return prop;
			} else {
				StringBuffer js = new StringBuffer();
				InputStreamReader inPutStream = (inStream != null) ? new InputStreamReader(inStream) : null;
				BufferedReader bufferedReader = (inPutStream != null) ? new BufferedReader(inPutStream) : null;
				if (bufferedReader != null) {
					String line = null;
					while ((line = bufferedReader.readLine()) != null) {
						js.append(line);
					}
					bufferedReader.close();
					System.out.println("  |--load " + configName + " OK Len=" + js.length());
				} else {
					System.out.println("  |--load " + configName + ",bufferedReader=" + bufferedReader);
				}
				return js;
			}
		} catch (Throwable e) {
			logger.error("load() error=" + e.toString());
			return null;
		} finally {
			if (inStream != null) {
				try {
					inStream.close();
				} catch (IOException e) {
					logger.error("load() error=" + e.toString());
				}
			}
		}
	}
}
