package com.newxtc.fp.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.newxtc.NxtInit;
import com.newxtc.fw.client.FwClient;
import com.newxtc.fw.client.util.AesUtil;
import com.newxtc.fw.client.util.CookieUtil;
import com.newxtc.fw.client.util.HttpClient;
import com.newxtc.fw.client.util.JsonParamUtil;
import com.newxtc.fw.client.util.MD5Util;
import com.newxtc.fw.client.util.WxAppUtil;

@WebServlet({ "/NxtJsServlet" })

public class NxtJsServlet extends HttpServlet {
	private static final long serialVersionUID = 1478580140226921020L;
	private final static Logger logger = LoggerFactory.getLogger(NxtJsServlet.class);
	private CookieUtil cookieUtil = new CookieUtil();
	private final static String fwServlet = "FwServlet";

	public NxtJsServlet() {
		super();
	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setHeader("P3P", "CP=CAO PSA OUR");
		response.setContentType("text/javascript;charset=UTF-8");
		PrintWriter out = response.getWriter();
		String mode = null;
		// 禁用设备指纹, 适用于风控系统不连接外网的情况, only suport
		String noFp = NxtInit.getInstance().getIni("noFp");
		if (noFp != null && "true".equals(noFp)) {
			out.print("console.log('noFp=" + noFp + ")");
			logger.debug("noFp=" + noFp);
			return;
		}
		String merchantCode = NxtInit.getInstance().getIni("merchantCode");
		String secKey = NxtInit.getInstance().getIni("secKey");
		try {
			String s_ = "";
			mode = request.getParameter("mode");
			String scheme = request.getParameter("scheme");
			String crypt = request.getParameter("crypt");
			// 获取当前服务器路径, scheme 参数为解决Nginx 转发后不一致的问题
			String baseUrl = (scheme != null ? scheme : request.getScheme()) + "://" + request.getServerName();
			int port = request.getServerPort();
			String uri = request.getRequestURI();
			baseUrl += (port != 80 && port != 443) ? ":" + port + uri : uri;

			if (null == mode || "".equals(mode)) {
				// 获取设备指纹的主URL
				StringBuffer mainUrl = new StringBuffer(baseUrl + "?mode=getFp");
				String expireUrl = baseUrl + "?mode=getExpire";
				if (scheme != null) {
					mainUrl.append("&scheme=" + scheme);
					expireUrl += "&scheme=" + scheme;
				}
				String style = request.getParameter("style");
				String callJs = request.getParameter("callJs");
				if (crypt != null && "aes".equals(crypt)) {
					mainUrl.append("&crypt=" + crypt);
				}
				if (style != null && !"".equals(style)) {
					mainUrl.append("&style=" + style);
				}
				if (callJs != null && !"".equals(callJs)) {
					mainUrl.append("&callJs=" + callJs);
				}
				Long t = System.currentTimeMillis();
				mainUrl.append("&t=" + t);
				// NXT_KEY = md5(merchantCode+seckey+t) , nav_s=md5(NXT_KEY+nav)
				String nxtKey = MD5Util.getMD5Str(merchantCode + secKey + t);
				StringBuffer js = new StringBuffer();
				js.append("var NXT_KEY='" + nxtKey + "';\n");
				js.append("var NXT_MAIN_URL='" + mainUrl + "';\n");
				js.append("var NXT_EXPIRE_URL='" + expireUrl + "';\n");
				String fpJs = NxtInit.getInstance().getfPJs();
				js.append(fpJs != null ? fpJs + "\n" : "");
				out.print(js.toString());
			} else {
				// 获取IP地址
				String clientIp = JsonParamUtil.getIpJson(request);
				// 加载远程 fingerJs
				switch (mode) {
				case "getFp":
					StringBuffer js = new StringBuffer();
					JSONObject jsonObj = getFrontJs(request, response, baseUrl, clientIp);
					String nxtJs = (jsonObj != null) ? jsonObj.getString("nxtJs") : null;
					if (nxtJs != null) {
						js.append(nxtJs);
					}
					if (crypt != null && "aes".equals(crypt)) {
						// 安全输入表单名称(with token)
						String cryptJs = getFrontCryptJs();
						js.append(cryptJs);
					}
					out.print(js.toString());
					break;
				case "frontCrypt":
					// 安全输入表单名称(单独进行加密)
					String cryptJs = getFrontCryptJs();
					out.print(cryptJs);
					break;
				case "callBack":
				case "getExpire":
					s_ = callOrExpire(request, response, mode, clientIp);
					out.print(s_);
					break;
				case "getDevice":
					String nxtToken = FwClient.getNxtToken(request);
					JSONObject deviceInfo = (nxtToken != null && nxtToken.length() >= 64) ? FwClient.getDevice(nxtToken) : null;
					if (deviceInfo != null) {
						String deviceId = deviceInfo.getString("deviceId");
						s_ = JSON.toJSONString(deviceInfo);
						logger.info("getDevice() tokenId=" + nxtToken.substring(0, 32) + "->" + deviceId);
					} else {
						logger.error("getDevice() null err,nxtToken=" + nxtToken + ",deviceInfo=" + deviceInfo);
					}
					out.print(s_);
					break;
				// 此方法也可以由业务系统实现
				case "getWxOpenId":
					String jsCode = request.getParameter("js_code");
					String sysInfo = request.getParameter("sys_info");
					String timezoneOffset = request.getParameter("timezone_off_set");
					s_ = WxAppUtil.getWxOpenId(jsCode, sysInfo, timezoneOffset, clientIp);
					out.print(s_);
					break;
				default:
					logger.error("err not exist mode=" + mode);
				}
			}
		} catch (Throwable e) {
			logger.error("mode=" + mode + ",error =" + e.toString());
			for (StackTraceElement elment : e.getStackTrace()) {
				logger.error(elment.toString());
			}
		} finally {
			out.flush();
			out.close();
		}
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}

	private JSONObject getFrontJs(HttpServletRequest request, HttpServletResponse response, String baseUrl, String clientIp) {
		JSONObject jsonObj = null;
		try {
			// 第一次加载， 为空
			String cookieNxtToken = cookieUtil.getCookie(request, JsonParamUtil.cookToken);
			String frontId = cookieUtil.getCookie(request, JsonParamUtil.cookFrontId);
			String t = request.getParameter("t");
			String backUrl = baseUrl + "?mode=callBack";
			String style = request.getParameter("style");
			String callJs = request.getParameter("callJs");
			String mode = request.getParameter("mode");
			String nav = request.getParameter("nav");
			String navSi = request.getParameter("navSi");
			String userAgent = request.getHeader("User-Agent");
			String referrer = JsonParamUtil.getReferrer(request);
			// 设置请求参数
			Map<String, Object> paramMap = new LinkedHashMap<String, Object>();
			paramMap.put("nxtToken", cookieNxtToken);
			paramMap.put("frontId", frontId);
			paramMap.put("mode", mode);
			paramMap.put("nav", nav);
			paramMap.put("navSi", navSi);
			paramMap.put("t", t);
			paramMap.put("referrer", referrer);
			paramMap.put("clientIp", clientIp);
			paramMap.put("userAgent", userAgent);
			if (style != null && !"".equals(style)) {
				paramMap.put("style", style);
			}
			if (callJs != null && !"".equals(callJs)) {
				paramMap.put("callJs", callJs);
			}
			paramMap.put("backUrl", backUrl);
			Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
			String jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", fwServlet, signParam, 50) : null;
			jsonObj = (jsonResp != null && jsonResp.startsWith("{") && jsonResp.endsWith("}")) ? JSONObject.parseObject(jsonResp) : null;
			if (jsonObj != null) {
				String nxtToken = (jsonObj != null) ? jsonObj.getString("nxtToken") : null;
				if (nxtToken != null && nxtToken.length() >= 64) {
					// 1 设置token cookie
					cookieUtil.setCookie(response, JsonParamUtil.cookToken, nxtToken);
				}
				String retFrontId = jsonObj.getString("frontId");
				if (retFrontId != null) {
					// 2 设置frontId cookie
					cookieUtil.setCookie(response, cookieUtil.getDomain(request), Integer.MAX_VALUE, JsonParamUtil.cookFrontId, retFrontId);
				}
				String nxtNav = jsonObj.getString("nxtNav");
				if (nxtNav != null) {
					// 3 设置nxtNav cookie
					cookieUtil.setCookie(response, JsonParamUtil.cookNav, nxtNav);
				}
			}
			return jsonObj;
		} catch (Exception e) {
			logger.error("getFrontJs() e=" + e.toString());
			for (StackTraceElement elment : e.getStackTrace()) {
				logger.error(elment.toString());
			}
			return null;
		}
	}

	private String callOrExpire(HttpServletRequest request, HttpServletResponse response, String mode, String clientIp) {
		String nxtToken = cookieUtil.getCookie(request, JsonParamUtil.cookToken);
		int tokenSize = (nxtToken != null) ? nxtToken.length() : 0;
		if (tokenSize < 64 || clientIp == null) {
			logger.error("callOrExpire() mode=" + mode + ",tokenSize(" + nxtToken + ")=" + tokenSize + ",clientIp=" + clientIp);
			return null;
		}
		String nxtNav = cookieUtil.getCookie(request, JsonParamUtil.cookNav);
		Map<String, Object> paramMap = new LinkedHashMap<>();
		paramMap.put("mode", mode);
		paramMap.put("nxtToken", nxtToken);
		paramMap.put("nxtNav", nxtNav);
		paramMap.put("clientIp", clientIp);
		Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
		String jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", fwServlet, paramMap, 500) : null;
		JSONObject callRet = (jsonResp != null && jsonResp.startsWith("{") && jsonResp.endsWith("}")) ? JSONObject.parseObject(jsonResp) : null;
		Integer ret = (callRet != null) ? callRet.getInteger("ret") : null;
		if (ret != null) {
			switch (ret) {
			case 0:
				String cookNxtToken = callRet.getString("cookNxtToken");
				if (cookNxtToken != null && cookNxtToken.length() >= 126) {
					// 设置token cookie
					cookieUtil.setCookie(response, JsonParamUtil.cookToken, nxtToken);
				} else {
					logger.error("callOrExpire() mode=" + mode + ",cookNxtToken=" + cookNxtToken + ",callRet=" + callRet.toJSONString() + ",clientIp=" + clientIp);
				}
				// 只在第一次返回时有值
				String retFrontId = callRet.getString("frontId");
				int frontLen = (retFrontId != null) ? retFrontId.length() : 0;
				if (frontLen >= 32) {
					// 2 设置frontId cookie
					cookieUtil.setCookie(response, cookieUtil.getDomain(request), Integer.MAX_VALUE, JsonParamUtil.cookFrontId, retFrontId);
					logger.debug("callOrExpire() mode=" + mode + ",frontId=" + retFrontId.substring(0, 32) + " ...(" + frontLen + ")" + ",clientIp=" + clientIp);
				}
				break;
			case -9:
				String errInfo = callRet.getString("err");
				logger.error("callOrExpire() mode=" + mode + "," + errInfo + ",clientIp=" + clientIp);
				break;
			default:
				logger.error("callOrExpire() mode=" + mode + "," + ret + ",not in (0,-9)" + ",clientIp=" + clientIp);
				return null;
			}
			return callRet.getString("msg");
		} else {
			logger.error("callOrExpire() mode=" + mode + ",jsonResp=" + jsonResp);
			return null;
		}
	}

	/**
	 * 获取加密JS
	 */
	private static String getFrontCryptJs() {
		StringBuffer js = new StringBuffer();
		String merchantCode = NxtInit.getInstance().getIni("merchantCode");
		String secKey = NxtInit.getInstance().getIni("secKey");
		// 安全输入表单名称
		String nxtAesKey = System.currentTimeMillis() + "";
		String nxtAesIv = MD5Util.getMD5Str(merchantCode + secKey + nxtAesKey);
		String nxtInputName = "NXT_" + MD5Util.getMD5Str(merchantCode + secKey + nxtAesKey + nxtAesIv);
		try {
			// 生成随机输入表单名称，
			Random random = new Random();
			int count = 20 + random.nextInt(31);
			StringBuilder mobileName = new StringBuilder("NXT_");
			char[] codes = JsonParamUtil.CODES;
			for (int i = 0, l = codes.length; i < count; i++) {
				mobileName.append(codes[random.nextInt(l)]);
			}
			String aesKey = AesUtil.convertTo16(nxtAesKey);
			String aesIv = AesUtil.convertTo16(nxtAesIv);
			String intPutKey = AesUtil.encrypt(mobileName.toString(), aesKey, aesIv);
			js.append("var nxtInputName=\"" + nxtInputName + "\";\n");
			js.append("var nxtInputKey=\"" + intPutKey + "\";\n");
			js.append("var nxtAesKey=\"" + nxtAesKey + "\";\n");
			js.append("var nxtAesIv=\"" + nxtAesIv + "\";\n");
			String desJs = NxtInit.getInstance().getDesJs();
			js.append(desJs);
			return js.toString();
		} catch (Exception e) {
			logger.error("getFrontCryptJs() nxtAesKey=" + nxtAesKey + ",nxtAesIv=" + nxtAesIv + ",nxtInputName=" + nxtInputName + " e=" + e.toString());
			return "";
		}
	}
}
