package com.newxtc.fw.client;

import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.time.FastDateFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.newxtc.fw.client.entity.EventParam;
import com.newxtc.fw.client.run.FwTask;
import com.newxtc.fw.client.run.FwThread;
import com.newxtc.fw.client.util.CookieUtil;
import com.newxtc.fw.client.util.HttpClient;
import com.newxtc.fw.client.util.JsonParamUtil;

public class FwClient {
	private final static Logger logger = LoggerFactory.getLogger(FwClient.class);
	private final static int outOfTime = 200;
	private final static FastDateFormat fm = FastDateFormat.getInstance("yyyy-MM-dd HH:mm:ss.SSS");

	private final static String fwServlet = "FwServlet";

	/**
	 * 1-1 下发短信
	 * 
	 * @param request
	 * @param phone
	 * @return
	 */
	public static Map<String, Object> getSendReq(HttpServletRequest request, String phone) {
		return EventParam.getMobileParam(request, "S_S", phone);
	}

	/**
	 * 1-2 下发短信
	 * 
	 * @param sJson
	 * @param phone
	 * @return
	 */

	public static Map<String, Object> getSendReq(String sJson, String phone) {
		return EventParam.getMobileParam(sJson, "S_S", phone);
	}

	/**
	 * 2-1 短信验证
	 * 
	 * @param request
	 * @param phone
	 * @return
	 */
	public static Map<String, Object> getVerifyReq(HttpServletRequest request, String phone) {
		return EventParam.getMobileParam(request, "S_V", phone);
	}

	/**
	 * 2-2 短信验证
	 * 
	 * @param sJson
	 * @param phone
	 * @return
	 */
	public static Map<String, Object> getVerifyReq(String sJson, String phone) {
		return EventParam.getMobileParam(sJson, "S_V", phone);
	}

	/**
	 * 3-1 手机号-密码登录
	 * 
	 * @param request
	 * @param phone
	 * @return
	 */
	public static Map<String, Object> getLoginByMobile(HttpServletRequest request, String phone) {
		return EventParam.getMobileParam(request, "U_L", phone);
	}

	/**
	 * 3-2 手机号-密码登录
	 * 
	 * @param sJson
	 * @param phone
	 * @return
	 */
	public static Map<String, Object> getLoginByMobile(String sJson, String phone) {
		return EventParam.getMobileParam(sJson, "U_L", phone);
	}

	/**
	 * 3-1 用户ID-密码登录
	 * 
	 * @param request
	 * @param userId
	 * @return
	 */
	public static Map<String, Object> getLoginByUid(HttpServletRequest request, String userId) {
		return EventParam.getUidParam(request, "U_L", userId);
	}

	/**
	 * 3-2 用户ID-密码登录
	 * 
	 * @param sJson
	 * @param userId
	 * @return
	 */
	public static Map<String, Object> getLoginByUid(String sJson, String userId) {
		return EventParam.getUidParam(sJson, "U_L", userId);
	}

	/**
	 * 4-1 email-密码登录
	 * 
	 * @param request
	 * @param userEmail
	 * @return
	 */
	public static Map<String, Object> getLoginByEmail(HttpServletRequest request, String userEmail) {
		return EventParam.getEmailParam(request, "U_L", userEmail);
	}

	/**
	 * 4-2 email-密码登录
	 * 
	 * @param request
	 * @param userEmail
	 * @return
	 */
	public static Map<String, Object> getLoginByEmail(String sJson, String userEmail) {
		return EventParam.getEmailParam(sJson, "U_L", userEmail);
	}

	public static JSONObject execReq(Map<String, Object> paramMap) {
		JSONObject retJson = null;
		Object eventType = null, status = null;
		try {
			Object objToken = (paramMap != null) ? paramMap.get("nxtToken") : null;
			String nxtToken = (objToken != null) ? objToken.toString() : null;
			String tokenId = "";
			if (nxtToken != null && nxtToken.length() >= 64) {
				eventType = paramMap.get("EVENT_TYPE");
				status = paramMap.get("status");
				tokenId = nxtToken.substring(0, 32);
			} else {
				logger.error("execReq() objToken=" + objToken);
				return retJson;
			}
			Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
			String jsonRes = (signParam != null) ? HttpClient.execClient("fireWareUrl", fwServlet, paramMap, outOfTime) : null;
			retJson = (jsonRes != null && jsonRes.startsWith("{") && jsonRes.endsWith("}")) ? JSON.parseObject(jsonRes) : null;
			if (retJson != null && retJson.size() > 0) {
				String orderNo = retJson.getString("orderNo");
				String riskResult = retJson.getString("riskResult");
				String ip = retJson.getString("ip");
				logger.info(eventType + "(" + status + "):" + tokenId + " -> " + orderNo + " risk=" + riskResult + ",ip=" + ip);
			} else {
				logger.error("execReq() Fw eventType=" + eventType + ",status=" + status + ",retJson=" + retJson);
			}
			return retJson;
		} catch (Throwable e) {
			// 系统异常 作放行处理
			logger.error("execReq() " + e.toString());
			return null;
		}
	}

	public static void execSucc(Map<String, Object> paramMap) {
		if (paramMap != null && paramMap.size() > 0) {
			Map<String, Object> succMap = new LinkedHashMap<String, Object>();
			succMap.putAll(paramMap);
			Long t = System.currentTimeMillis();
			succMap.put("finish_time", fm.format(t));
			succMap.put("status", "1");
			FwThread.getInstance().addTask(new FwTask(succMap));
		} else {
			logger.error("execSucc() paramMap=" + paramMap);
		}
	}

	public static void execFail(Map<String, Object> paramMap) {
		if (paramMap != null && paramMap.size() > 0) {
			Map<String, Object> failMap = new LinkedHashMap<String, Object>();
			failMap.putAll(paramMap);
			Long t = System.currentTimeMillis();
			failMap.put("finish_time", fm.format(t));
			failMap.put("status", "-1");
			FwThread.getInstance().addTask(new FwTask(failMap));
		} else {
			logger.error("execSucc() paramMap=" + paramMap);
		}
	}

	public static JSONObject getDevice(HttpServletRequest request) {
		String nxtToken = getNxtToken(request);
		if (nxtToken != null && nxtToken.length() >= 64) {
			return getDevice(nxtToken);
		} else {
			String clientIp = JsonParamUtil.getIpJson(request);
			logger.error("getDeviceInfo() error nxtToken=" + nxtToken + ",clientIp=" + clientIp);
			return null;
		}
	}

	public static JSONObject getDevice(String nxtToken) {
		String jsonResp = null;
		if (nxtToken != null && nxtToken.length() >= 64) {
			try {
				Map<String, Object> paramMap = new LinkedHashMap<>();
				JSONObject deviceInfo = null;
				paramMap.put("mode", "getDevice");
				paramMap.put("nxtToken", nxtToken);
				Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
				jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", fwServlet, paramMap, 100) : null;
				deviceInfo = (jsonResp != null && jsonResp.contains("{") && jsonResp.contains("}")) ? JSONObject.parseObject(jsonResp, Feature.OrderedField) : null;
				return deviceInfo;
			} catch (Exception e) {
				logger.error("getDevice() nxtToken=" + nxtToken + ",jsonResp=" + jsonResp + " e=" + e.toString());
				return null;
			}
		} else {
			logger.error("getDevice() token is null");
			return null;
		}
	}

	public static String getNxtToken(HttpServletRequest request) {
		CookieUtil cookieUtil = new CookieUtil();
		String nxtToken = request.getParameter("nxtToken");
		nxtToken = (nxtToken != null && nxtToken.length() >= 64) ? nxtToken : cookieUtil.getCookie(request, JsonParamUtil.cookToken);
		if (nxtToken != null && nxtToken.length() >= 64) {
			return nxtToken.substring(0, 64);
		} else {
			return null;
		}
	}
}
