package com.newxtc.fw.client;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.fw.client.util.HttpClient;
import com.newxtc.fw.client.util.JsonParamUtil;
import com.newxtc.fw.client.util.WxAppUtil;

public class SmsSafe {

	private final static Logger logger = LoggerFactory.getLogger(SmsSafe.class);

	/**
	 * 1-1 自定义安全短信(request请求)
	 * 
	 * @param phone
	 * @param msg
	 * @param request
	 * @return
	 */
	public static JSONObject sendMsg(String phone, String msg, HttpServletRequest request) {
		JSONObject jsonObj = JsonParamUtil.getRequestJson(request);
		return smsSend(phone, null, null, msg, jsonObj);
	}

	/**
	 * 1-2 自定义安全短信(json)
	 * 
	 * @param phone
	 * @param msg
	 * @param inputJson
	 * @return
	 */
	public static JSONObject sendMsg(String phone, String msg, String inputJson) {
		JSONObject jsonObj = (inputJson != null && inputJson.contains("{") && inputJson.contains("}")) ? JSONObject.parseObject(inputJson) : null;
		return smsSend(phone, null, null, msg, jsonObj);
	}

	/**
	 * 1-3 自定义安全短信(微信端)
	 * 
	 * @param phone
	 * @param msg
	 * @param nxtData
	 * @return
	 */
	public static JSONObject sendMsgWx(String phone, String msg, String nxtData) {
		JSONObject wxJson = WxAppUtil.getWxJson(nxtData);
		return smsSend(phone, null, null, msg, wxJson);
	}

	/**
	 * 2-1 模板方式安全短信(request请求)
	 * 
	 * @param phone
	 * @param templateId
	 * @param templateMap
	 * @param request
	 * @return
	 */
	public static JSONObject sendTpMsg(String phone, String templateId, Map<String, String> templateMap, HttpServletRequest request) {
		JSONObject jsonObj = JsonParamUtil.getRequestJson(request);
		return smsSend(phone, templateId, templateMap, null, jsonObj);
	}

	/**
	 * 2-2 模板方式安全短信(json)
	 * 
	 * @param phone
	 * @param templateId
	 * @param templateMap
	 * @param inputJson
	 * @return
	 */
	public static JSONObject sendTpMsg(String phone, String templateId, Map<String, String> templateMap, String inputJson) {
		JSONObject jsonObj = (inputJson != null && inputJson.startsWith("{") && inputJson.endsWith("}")) ? JSONObject.parseObject(inputJson) : null;
		return smsSend(phone, templateId, templateMap, null, jsonObj);
	}

	/**
	 * 2-3 模板方式安全短信(微信小程序)
	 * 
	 * @param phone
	 * @param templateId
	 * @param templateMap
	 * @param nxtData
	 *            (wx_open_id,wx_sys_info,wx_js_code,wx_sign)
	 * @return
	 */
	public static JSONObject sendTpMsgWx(String phone, String templateId, Map<String, String> templateMap, String nxtData) {
		JSONObject wxJson = WxAppUtil.getWxJson(nxtData);
		return smsSend(phone, templateId, templateMap, null, wxJson);
	}

	private static JSONObject smsSend(String phone, String templateId, Map<String, String> templateMap, String msg, JSONObject jsonObj) {
		String jsonResp = null;
		Map<String, Object> paramMap = JsonParamUtil.getParamByJson(jsonObj);
		try {
			String templateJson = null;
			if (phone != null && phone.length() == 11) {
				// 2 组装短信发送参数
				paramMap.put("phone", phone);
				if (templateId != null && !"".equals(templateId)) {
					templateJson = (templateMap != null) ? JSONObject.toJSONString(templateMap) : null;
					paramMap.put("templateId", templateId);
					paramMap.put("templateJson", templateJson);
				} else if (msg != null) {
					paramMap.put("msg", msg);
				} else {
					logger.error("smsSend() both msg:" + msg + " and templateId:" + templateId + " is null");
					return null;
				}
			} else {
				logger.error("smsSend() phone=" + phone);
				return null;
			}

			StringBuffer sbLog = new StringBuffer();
			sbLog.append("phone=" + phone);
			// wx app
			String openid = jsonObj.getString("openid");
			if (openid != null) {
				paramMap.put("wx_open_id", openid);
				sbLog.append(",openid=" + openid);
			}
			String sysInfo = jsonObj.getString("sys_info");
			if (sysInfo != null && sysInfo.contains("{") && sysInfo.contains("}")) {
				paramMap.put("sys_info", sysInfo);
			}
			if (templateId != null) {
				sbLog.append(",templateId=" + templateId + ",templateJson=" + templateJson);
			} else {
				sbLog.append(",msg=" + msg);
			}
			Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
			jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", "SmsSafeServlet", signParam, 1000) : null;
			JSONObject retJson = (jsonResp != null) ? JSONObject.parseObject(jsonResp) : null;
			int ret = (retJson != null) ? retJson.getIntValue("ret") : -1;
			if (ret == 0) {
				String spCode = (retJson != null) ? retJson.getString("spCode") : null;
				logger.info("smsSend() spCode=" + spCode + "," + sbLog);
			} else if (ret == 99999) {
				// 被短信防火墙拦截
				Object ipObj = paramMap.get("clientIp");
				String ip = (ipObj != null) ? (String) ipObj : "";
				String riskResult = retJson.getString("riskResult");
				logger.info("smsSend() orderNo:" + retJson.getString("orderNo") + ",risk=" + riskResult + ",ip=" + ip + "," + sbLog);
			} else {
				logger.error("smsSend() ret=" + ret + ",jsonResp=" + jsonResp + "," + sbLog);
			}
			return retJson;
		} catch (Exception e) {
			logger.error("smsSend() paramMap=" + paramMap + ",jsonResp=" + jsonResp);
			return null;
		}
	}
}
