package com.newxtc.fw.client;

import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.time.FastDateFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.NxtInit;
import com.newxtc.fw.client.util.AesUtil;
import com.newxtc.fw.client.util.Base64;
import com.newxtc.fw.client.util.JsonParamUtil;
import com.newxtc.fw.client.util.MD5Util;

/**
 * 手机号加解密插件 aes 算法
 * 
 * @author topliu
 *
 */
public class WebCrypt {
	private final static Logger logger = LoggerFactory.getLogger(WebCrypt.class);

	private final static FastDateFormat fm = FastDateFormat.getInstance("yyyy-MM-dd HH:mm:ss.SSS");

	/**
	 * 一步获取 支持标准 request.getParameter 表单获取方式
	 * 
	 * @param request
	 * @param response
	 * @return
	 */

	public static String getMobile(HttpServletRequest request) {
		JSONObject inputJson = new JSONObject();
		Enumeration<String> enu = request.getParameterNames();
		String inputName, inputValue;
		while (enu.hasMoreElements()) {
			inputName = enu.nextElement();
			inputValue = request.getParameter(inputName);
			inputJson.put(inputName, inputValue);
		}
		String clientIp = JsonParamUtil.getIpJson(request);
		if (clientIp != null) {
			inputJson.put("clientIp", clientIp);
		} else {
			logger.error("getMobile() clientIp=" + clientIp);
		}
		return mobileDec(inputJson);
	}

	/**
	 * 一步获取手机号 支持 JSON 获取方式, 一般应用在 vue 结构
	 * 
	 * @param request
	 * @param inputJson
	 * @return
	 */

	public static String getMobileByJson(String inputJson) {
		try {
			JSONObject jsonObj = (inputJson != null && inputJson.contains("{") && inputJson.contains("}")) ? JSONObject.parseObject(inputJson) : null;
			return mobileDec(jsonObj);
		} catch (Exception e) {
			logger.error("getMobileByJson() e=" + e.toString());
			return "";
		}
	}

	/**
	 * 手机号解密,
	 * 
	 * @param request
	 * @param jsonObj
	 * @return
	 */
	private static String mobileDec(JSONObject jsonObj) {
		String merchantCode = NxtInit.getInstance().getIni("merchantCode");
		String secKey = NxtInit.getInstance().getIni("secKey");
		String mobile = "";
		if (jsonObj == null) {
			logger.error("mobileDec() jsonObj=" + jsonObj);
			return "";
		}
		String aesKey = jsonObj.getString("nxtAesKey");
		String aesIv = jsonObj.getString("nxtAesIv");
		String nxtMobile = jsonObj.getString("nxtMobile");
		String nxtMobileSi = jsonObj.getString("nxtMobileSi");
		String ip = jsonObj.getString("clientIp");
		if (aesKey != null) {
			Long aesKeyL = Long.parseLong(aesKey);
			// 0 判断时间>10分钟
			Long timeOff = Math.abs(System.currentTimeMillis() - aesKeyL);
			if (timeOff.compareTo(600000L) > 0) {
				logger.error("mobileDec() timeOff=" + timeOff + ",aesKey=" + aesKey + " -> " + fm.format(aesKeyL) + ",ip=" + ip);
				return "";
			}
		} else {
			logger.error("mobileDec() aesKey=" + aesKey + ",jsonObj=" + jsonObj.toJSONString());
			return "";
		}
		StringBuffer errSb = new StringBuffer();
		String sign = (aesIv != null) ? MD5Util.getMD5Str(merchantCode + secKey + aesKey) : null;
		if (aesIv.equals(sign)) {
			// 1 获取安全密钥表单名称
			String nxtInputName = "NXT_" + MD5Util.getMD5Str(merchantCode + secKey + aesKey + aesIv);
			// 2 从request 获取安全密钥
			String nxtInputKey = jsonObj.getString(nxtInputName);
			// 3 获取手机号表单名称
			String mobileName = (nxtInputKey != null) ? AesUtil.aesDec(aesKey, aesIv, nxtInputKey) : null;
			// 4 获取手机号密文
			String mobileEnc = (mobileName != null) ? jsonObj.getString(mobileName) : null;
			// 5 解密手机号
			String mobileTxt = (mobileEnc != null) ? AesUtil.aesDec(aesKey, aesIv, mobileEnc) : null;
			if (mobileTxt != null && !"".equals(mobileTxt)) {
				mobile = mobileTxt;
			} else {
				if (nxtInputKey == null)
					errSb.append(",1:nxtInputKey(nxtInputName:" + nxtInputName + ")=" + nxtInputKey);
				if (mobileName == null)
					errSb.append(",2:mobileName=" + mobileName);
				if (mobileEnc == null)
					errSb.append(",3:mobileEnc=" + mobileEnc);
			}
		}
		if ((mobile == null || "".equals(mobile)) && nxtMobile != null && nxtMobileSi != null) {
			String mobileTxt = (nxtMobile != null) ? Base64.decodeString(nxtMobile) : null;
			sign = (mobileTxt != null && !"".equals(mobileTxt)) ? MD5Util.getMD5Str(aesKey + mobileTxt) : null;
			if (sign != null && sign.equals(nxtMobileSi)) {
				mobile = mobileTxt;
				errSb.append(",4:base64 succ");
			}
		}
		if (errSb.length() > 0) {
			logger.error("mobileDec() dec err" + errSb.toString() + ",ip=" + ip);
		}
		return mobile;
	}
}
