﻿package com.newxtc.fw.client.entity;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.time.FastDateFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.fw.client.util.JsonParamUtil;
import com.newxtc.fw.client.util.WxAppUtil;

public class EventParam {

	private final static Logger logger = LoggerFactory.getLogger(EventParam.class);

	private final static FastDateFormat fm = FastDateFormat.getInstance("yyyy-MM-dd HH:mm:ss.SSS");

	// mobile-request
	public static Map<String, Object> getMobileParam(HttpServletRequest request, String eventType, String phone) {
		if (phone != null && phone.length() == 11) {
			JSONObject jsonObj = JsonParamUtil.getRequestJson(request);
			return getParamByObj(jsonObj, eventType, phone, null, phone);
		} else {
			logger.error("getMobileParam() phone=" + phone);
			return null;
		}
	}

	// mobile-json
	public static Map<String, Object> getMobileParam(String inputJson, String eventType, String phone) {
		if (phone != null && phone.length() == 11) {
			JSONObject jsonObj = getJsonObj(inputJson);
			return getParamByObj(jsonObj, eventType, phone, null, phone);
		} else {
			logger.error("getMobileParam() phone=" + phone);
			return null;
		}
	}

	// mobile-wx
	public static Map<String, Object> getMobileParamWx(String wxData, String eventType, String phone) {
		if (phone != null && phone.length() == 11) {
			JSONObject jsonObj = WxAppUtil.getWxJson(wxData);
			return getParamByObj(jsonObj, eventType, phone, null, phone);
		} else {
			logger.error("getMobileParamWx() phone=" + phone);
			return null;
		}
	}

	// uid-request
	public static Map<String, Object> getUidParam(HttpServletRequest request, String eventType, String userId) {
		if (userId != null && !"".equals(userId)) {
			JSONObject jsonObj = JsonParamUtil.getRequestJson(request);
			return getParamByObj(jsonObj, eventType, null, null, userId);
		} else {
			logger.error("getUidParam() userId=" + userId);
			return null;
		}
	}

	// uid-json
	public static Map<String, Object> getUidParam(String inputJson, String eventType, String userId) {
		if (userId != null && !"".equals(userId)) {
			JSONObject jsonObj = getJsonObj(inputJson);
			return getParamByObj(jsonObj, eventType, null, null, userId);
		} else {
			logger.error("getUidParam() userId=" + userId);
			return null;
		}
	}

	// email-request
	public static Map<String, Object> getEmailParam(HttpServletRequest request, String eventType, String userEmail) {
		if (userEmail != null && !"".equals(userEmail)) {
			JSONObject jsonObj = JsonParamUtil.getRequestJson(request);
			return getParamByObj(jsonObj, eventType, null, userEmail, userEmail);
		} else {
			logger.error("getEmailParam() userEmail=" + userEmail);
			return null;
		}
	}

	// email-json
	public static Map<String, Object> getEmailParam(String inputJson, String eventType, String userEmail) {
		if (userEmail != null && !"".equals(userEmail)) {
			JSONObject jsonObj = getJsonObj(inputJson);
			return getParamByObj(jsonObj, eventType, null, userEmail, userEmail);
		} else {
			logger.error("getEmailParam() userEmail=" + userEmail);
			return null;
		}
	}

	private static JSONObject getJsonObj(String inputJson) {
		try {
			JSONObject jsonObj = (inputJson != null && inputJson.startsWith("{") && inputJson.endsWith("}")) ? JSONObject.parseObject(inputJson) : null;
			return jsonObj;
		} catch (Exception e) {
			logger.error("getJsonObj() inputJson=" + inputJson + ",e=" + e.toString());
			return null;
		}
	}

	/**
	 * @param request
	 * @param userId
	 * @param inputMap
	 */
	private static Map<String, Object> getParamByObj(JSONObject jsonObj, String eventType, String phone, String userEmail, String userId) {
		Map<String, Object> paramMap = JsonParamUtil.getParamByJson(jsonObj);
		if (paramMap != null) {
			paramMap.put("mode", "fw");
			paramMap.put("EVENT_TYPE", eventType);
			paramMap.put("status", "0");// 首次请求状态参数，必填
			paramMap.put("occur_time", fm.format(System.currentTimeMillis()));// 发生事件，必填

			if (phone != null) {
				paramMap.put("phone", phone);
			}
			if (userEmail != null) {
				paramMap.put("client_email", userEmail);
			}
			if (userId != null) {
				paramMap.put("user_id", userId);
			}
			return paramMap;
		} else {
			logger.error("getParamByObj() paramMap=" + paramMap);
			return null;
		}
	}
}
